@@ -18,10 +18,13 @@ from networking_odl.common import constants as odl_const
 from networking_odl.common import filters
 from networking_odl.db import db
 from networking_odl.journal import journal
-from networking_odl.l3 import l3_odl_v2
+from networking_odl.journal import maintenance
 from networking_odl.ml2 import mech_driver_v2
+from networking_odl.tests import base as odl_base
+from networking_odl.tests.unit import test_base_db
 
 import mock
+from oslo_config import cfg
 from oslo_serialization import jsonutils
 import requests
 
@@ -29,7 +32,7 @@ from neutron import context
 from neutron.db import api as neutron_db_api
 from neutron.extensions import external_net as external_net
 from neutron import manager
-from neutron.plugins.ml2 import config as config
+from neutron.plugins.common import constants as service_constants
 from neutron.plugins.ml2 import plugin
 from neutron.tests import base
 from neutron.tests.unit.db import test_db_base_plugin_v2
@@ -44,20 +47,25 @@ PORT_ID = 'port_uuid'
 
 
 class OpenDayLightMechanismConfigTests(testlib_api.SqlTestCase):
+    def setUp(self):
+        super(OpenDayLightMechanismConfigTests, self).setUp()
+        cfg.CONF.set_override('mechanism_drivers',
+                              ['logger', 'opendaylight_v2'], 'ml2')
+        cfg.CONF.set_override('port_binding_controller',
+                              'legacy-port-binding', 'ml2_odl')
 
     def _set_config(self, url='http://127.0.0.1:9999', username='someuser',
                     password='somepass'):
-        config.cfg.CONF.set_override('mechanism_drivers',
-                                     ['logger', 'opendaylight'],
-                                     'ml2')
-        config.cfg.CONF.set_override('url', url, 'ml2_odl')
-        config.cfg.CONF.set_override('username', username, 'ml2_odl')
-        config.cfg.CONF.set_override('password', password, 'ml2_odl')
+        cfg.CONF.set_override('url', url, 'ml2_odl')
+        cfg.CONF.set_override('username', username, 'ml2_odl')
+        cfg.CONF.set_override('password', password, 'ml2_odl')
 
     def _test_missing_config(self, **kwargs):
         self._set_config(**kwargs)
-        self.assertRaises(config.cfg.RequiredOptError,
-                          plugin.Ml2Plugin)
+        self.assertRaisesRegex(cfg.RequiredOptError,
+                               'value required for option \w+ in group '
+                               '\[ml2_odl\]',
+                               plugin.Ml2Plugin)
 
     def test_valid_config(self):
         self._set_config()
@@ -78,43 +86,49 @@ class DataMatcher(object):
     def __init__(self, operation, object_type, object_dict):
         self._data = object_dict.copy()
         self._object_type = object_type
-        filter_cls = filters.FILTER_MAP[object_type]
-        attr_filter = getattr(filter_cls,
-                              'filter_' + operation + '_attributes')
-        attr_filter(self._data)
+        filters.filter_for_odl(object_type, operation, self._data)
 
     def __eq__(self, s):
         data = jsonutils.loads(s)
-        if self._object_type == odl_const.ODL_ROUTER_INTF:
-            return self._data == data
-        else:
-            return self._data == data[self._object_type]
+        return self._data == data[self._object_type]
+
+    def __ne__(self, s):
+        return not self.__eq__(s)
 
 
 class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
+                             test_base_db.ODLBaseDbTestCase,
                              base.BaseTestCase):
     def setUp(self):
-        config.cfg.CONF.set_override("core_plugin",
-                                     'neutron.plugins.ml2.plugin.Ml2Plugin')
-        core_plugin = config.cfg.CONF.core_plugin
-        super(OpenDaylightL3TestCase, self).setUp(plugin=core_plugin)
-        config.cfg.CONF.set_override('mechanism_drivers',
-                                     ['logger', 'opendaylight'], 'ml2')
-        config.cfg.CONF.set_override('url', 'http://127.0.0.1:9999', 'ml2_odl')
-        config.cfg.CONF.set_override('username', 'someuser', 'ml2_odl')
-        config.cfg.CONF.set_override('password', 'somepass', 'ml2_odl')
+        cfg.CONF.set_override("core_plugin",
+                              'neutron.plugins.ml2.plugin.Ml2Plugin')
+        cfg.CONF.set_override('mechanism_drivers',
+                              ['logger', 'opendaylight_v2'], 'ml2')
+        self.useFixture(odl_base.OpenDaylightRestClientFixture())
+        cfg.CONF.set_override("service_plugins", ['odl-router_v2'])
+        core_plugin = cfg.CONF.core_plugin
+        service_plugins = {'l3_plugin_name': 'odl-router_v2'}
         mock.patch.object(journal.OpendaylightJournalThread,
                           'start_odl_sync_thread').start()
+        self.mock_mt_thread = mock.patch.object(
+            maintenance.MaintenanceThread, 'start').start()
+        mock.patch.object(mech_driver_v2.OpenDaylightMechanismDriver,
+                          '_record_in_journal').start()
+        mock.patch.object(mech_driver_v2.OpenDaylightMechanismDriver,
+                          'sync_from_callback_precommit').start()
+        mock.patch.object(mech_driver_v2.OpenDaylightMechanismDriver,
+                          'sync_from_callback_postcommit').start()
+        super(OpenDaylightL3TestCase, self).setUp(
+            plugin=core_plugin, service_plugins=service_plugins)
         self.db_session = neutron_db_api.get_session()
-        self.mech = mech_driver_v2.OpenDaylightMechanismDriver()
         self.plugin = manager.NeutronManager.get_plugin()
         self.plugin._network_is_external = mock.Mock(return_value=True)
-        self.driver = l3_odl_v2.OpenDaylightL3RouterPlugin()
+        self.driver = manager.NeutronManager.get_service_plugins()[
+            service_constants.L3_ROUTER_NAT]
         self.thread = journal.OpendaylightJournalThread()
         self.driver.get_floatingip = mock.Mock(
             return_value={'router_id': ROUTER_ID,
                           'floating_network_id': NETWORK_ID})
-        self.addCleanup(self._db_cleanup)
 
     @staticmethod
     def _get_mock_router_operation_info(network, subnet):
@@ -147,11 +161,6 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
         getter = getattr(cls, '_get_mock_' + object_type + '_operation_info')
         return getter(*args)
 
-    def _db_cleanup(self):
-        rows = db.get_all_db_rows(self.db_session)
-        for row in rows:
-            db.delete_row(self.db_session, row=row)
-
     @classmethod
     def _get_mock_request_response(cls, status_code):
         response = mock.Mock(status_code=status_code)
@@ -171,9 +180,9 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
         if expected_calls:
             mock_method.assert_called_with(
                 headers={'Content-Type': 'application/json'},
-                auth=(config.cfg.CONF.ml2_odl.username,
-                      config.cfg.CONF.ml2_odl.password),
-                timeout=config.cfg.CONF.ml2_odl.timeout, *args, **kwargs)
+                auth=(cfg.CONF.ml2_odl.username,
+                      cfg.CONF.ml2_odl.password),
+                timeout=cfg.CONF.ml2_odl.timeout, *args, **kwargs)
         self.assertEqual(expected_calls, mock_method.call_count)
 
     def _call_operation_object(self, operation, object_type, object_id,
@@ -186,10 +195,6 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
             new_object_dict = method(object_context, object_dict)
         elif operation == odl_const.ODL_UPDATE:
             new_object_dict = method(object_context, object_id, object_dict)
-        elif operation in [odl_const.ODL_ADD, odl_const.ODL_REMOVE]:
-            router_dict = method(object_context, object_id, object_dict)
-            new_object_dict = self.driver._generate_router_dict(
-                object_id, object_dict, router_dict)
         else:
             new_object_dict = method(object_context, object_id)
 
@@ -200,14 +205,10 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
                                           expected_calls=1):
         http_requests = {odl_const.ODL_CREATE: 'post',
                          odl_const.ODL_UPDATE: 'put',
-                         odl_const.ODL_DELETE: 'delete',
-                         odl_const.ODL_ADD: 'put',
-                         odl_const.ODL_REMOVE: 'put'}
+                         odl_const.ODL_DELETE: 'delete'}
         status_codes = {odl_const.ODL_CREATE: requests.codes.created,
                         odl_const.ODL_UPDATE: requests.codes.ok,
-                        odl_const.ODL_DELETE: requests.codes.no_content,
-                        odl_const.ODL_ADD: requests.codes.created,
-                        odl_const.ODL_REMOVE: requests.codes.created}
+                        odl_const.ODL_DELETE: requests.codes.no_content}
 
         http_request = http_requests[operation]
         status_code = status_codes[operation]
@@ -218,16 +219,12 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
 
         # Setup expected results.
         if operation in [odl_const.ODL_UPDATE, odl_const.ODL_DELETE]:
-            url = (config.cfg.CONF.ml2_odl.url + '/' + object_type + 's/' +
+            url = (cfg.CONF.ml2_odl.url + '/' + object_type + 's/' +
                    object_id)
-        elif operation in [odl_const.ODL_ADD, odl_const.ODL_REMOVE]:
-            url = (config.cfg.CONF.ml2_odl.url + '/' + odl_const.ODL_ROUTER +
-                   's/' + object_id + '/' + operation + '_router_interface')
         else:
-            url = config.cfg.CONF.ml2_odl.url + '/' + object_type + 's'
+            url = cfg.CONF.ml2_odl.url + '/' + object_type + 's'
 
-        if operation in [odl_const.ODL_CREATE, odl_const.ODL_UPDATE,
-                         odl_const.ODL_ADD, odl_const.ODL_REMOVE]:
+        if operation in [odl_const.ODL_CREATE, odl_const.ODL_UPDATE]:
             kwargs = {
                 'url': url,
                 'data': DataMatcher(operation, object_type, new_object_dict)}
@@ -354,78 +351,12 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
     def test_floatingip_db(self):
         self._test_object_db(odl_const.ODL_FLOATINGIP)
 
-    def test_router_intf_db(self):
-        # Create network, subnet and router for testing.
-        kwargs = {'arg_list': (external_net.EXTERNAL,),
-                  external_net.EXTERNAL: True}
-        with self.network(**kwargs) as network:
-            with self.subnet(cidr='10.0.0.0/24') as subnet:
-                router_context, router_dict = (
-                    self._get_mock_router_operation_info(network, None))
-                new_router_dict = self.driver.create_router(router_context,
-                                                            router_dict)
-                router_id = new_router_dict['id']
-
-                object_type = odl_const.ODL_ROUTER_INTF
-                router_intf_context, router_intf_dict = \
-                    self._get_mock_router_interface_operation_info(network,
-                                                                   subnet)
-
-                # Remove 'router' database entry to allow tests to pass.
-                self._db_cleanup()
-
-                # Add and test router interface 'add' database entry.
-                # Note that router interface events do not generate unique
-                # UUIDs.
-                self.driver.add_router_interface(router_intf_context,
-                                                 router_id, router_intf_dict)
-                self._test_db_results(odl_const.ODL_UUID_NOT_USED,
-                                      odl_const.ODL_ADD, object_type)
-
-                # Add and test 'remove' database entry.
-                self.driver.remove_router_interface(router_intf_context,
-                                                    router_id,
-                                                    router_intf_dict)
-                self._test_db_results(odl_const.ODL_UUID_NOT_USED,
-                                      odl_const.ODL_REMOVE, object_type)
-
     def test_router_threading(self):
         self._test_thread_processing(odl_const.ODL_ROUTER)
 
     def test_floatingip_threading(self):
         self._test_thread_processing(odl_const.ODL_FLOATINGIP)
 
-    def test_router_intf_threading(self):
-        # Create network, subnet and router for testing.
-        kwargs = {'arg_list': (external_net.EXTERNAL,),
-                  external_net.EXTERNAL: True}
-        with self.network(**kwargs) as network:
-            with self.subnet(cidr='10.0.0.0/24') as subnet:
-                router_context, router_dict = (
-                    self._get_mock_router_operation_info(network, None))
-                new_router_dict = self.driver.create_router(router_context,
-                                                            router_dict)
-                router_id = new_router_dict['id']
-                object_type = odl_const.ODL_ROUTER_INTF
-
-                # Add and process router interface 'add' request. Adds to
-                # database. Expected calls = 2 because the create_router db
-                # entry is also processed.
-                self._test_operation_thread_processing(
-                    object_type, odl_const.ODL_ADD, network, subnet, router_id,
-                    expected_calls=2)
-                rows = db.get_all_db_rows_by_state(self.db_session,
-                                                   odl_const.COMPLETED)
-                self.assertEqual(2, len(rows))
-
-                # Add and process 'remove' request. Adds to database.
-                self._test_operation_thread_processing(
-                    object_type, odl_const.ODL_REMOVE, network, subnet,
-                    router_id)
-                rows = db.get_all_db_rows_by_state(self.db_session,
-                                                   odl_const.COMPLETED)
-                self.assertEqual(3, len(rows))
-
     def test_delete_network_validate_ext_delete_router_dep(self):
         router_context = [NETWORK_ID]
         self._test_dependency_processing(
@@ -447,13 +378,6 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
             odl_const.ODL_DELETE, odl_const.ODL_FLOATINGIP, FLOATINGIP_ID,
             floatingip_context)
 
-    def test_delete_router_validate_ext_remove_routerintf_dep(self):
-        router_intf_dict = {'id': ROUTER_ID}
-        self._test_dependency_processing(
-            odl_const.ODL_DELETE, odl_const.ODL_ROUTER, ROUTER_ID, None,
-            odl_const.ODL_REMOVE, odl_const.ODL_ROUTER_INTF,
-            odl_const.ODL_UUID_NOT_USED, router_intf_dict)
-
     def test_delete_router_validate_self_create_dep(self):
         self._test_dependency_processing(
             odl_const.ODL_DELETE, odl_const.ODL_ROUTER, ROUTER_ID, EMPTY_DEP,
@@ -499,28 +423,3 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
             EMPTY_DEP,
             odl_const.ODL_UPDATE, odl_const.ODL_FLOATINGIP, FLOATINGIP_ID,
             None)
-
-    def test_add_router_intf_validate_ext_create_router_dep(self):
-        router_intf_context = {'subnet_id': SUBNET_ID,
-                               'id': ROUTER_ID}
-        self._test_dependency_processing(
-            odl_const.ODL_ADD, odl_const.ODL_ROUTER_INTF,
-            odl_const.ODL_UUID_NOT_USED, router_intf_context,
-            odl_const.ODL_CREATE, odl_const.ODL_ROUTER, ROUTER_ID, None)
-
-    def test_add_router_intf_validate_ext_create_subnet_dep(self):
-        router_intf_context = {'subnet_id': SUBNET_ID,
-                               'id': ROUTER_ID}
-        self._test_dependency_processing(
-            odl_const.ODL_ADD, odl_const.ODL_ROUTER_INTF,
-            odl_const.ODL_UUID_NOT_USED, router_intf_context,
-            odl_const.ODL_CREATE, odl_const.ODL_SUBNET, SUBNET_ID, None)
-
-    def test_remove_router_intf_validate_self_remove_router_intf_dep(self):
-        router_intf_context = {'subnet_id': SUBNET_ID,
-                               'id': ROUTER_ID}
-        self._test_dependency_processing(
-            odl_const.ODL_REMOVE, odl_const.ODL_ROUTER_INTF,
-            odl_const.ODL_UUID_NOT_USED, router_intf_context,
-            odl_const.ODL_ADD, odl_const.ODL_ROUTER_INTF,
-            odl_const.ODL_UUID_NOT_USED, router_intf_context)
